import { jsx as _jsx, Fragment as _Fragment, jsxs as _jsxs } from "react/jsx-runtime";
import React, { useState } from 'react';
import { Visibility, VisibilityOff } from '@mui/icons-material';
import { Container, IconButton, InputAdornment, InputLabel, MenuItem, Select, TextField, Typography, } from '@mui/material';
import { INPUT_LABELS, VariantTypes } from '../../constants/common-constants';
import { COUNTRIES } from '../../constants/countries';
import { InputAdornmentPosition, InputAutoCompleteType, InputIconPosition, InputPresets, InputSizes, InputType, NumberType, } from '../../constants/input-field-constants';
import { NO_SPACE_REGEX, ONLY_NUMBER_REGEX, PARSE_DECIMAL_REGEX, } from '../../constants/regex-constants';
import './adminInput-styles.scss';
const AdminInput = (props) => {
    const { disabled = false, autoComplete = InputAutoCompleteType.ON, containerClass = '', helperText = 'Oops! Wrong input', onChange = () => { }, hasBorder = true, icon, iconPosition, inputFieldClass = '', inputFieldWithBorderClass = '', inputLabelClass = '', inputLabelWithBorderClass = '', error = false, isErrorBorder = true, isFullWidth = true, isLabelError = true, isMultiline = false, required, label, maxLength = 0, placeholder, preset = InputPresets.Text, size = InputSizes.Small, value, variant = preset === InputPresets.Tel ? VariantTypes.Standard : VariantTypes.Outlined, maxLines = 1, name, numberType = NumberType.Integer, ...rest } = props;
    const [showPassword, setShowPassword] = useState(false);
    const [countryCode, phoneNumber] = String(value)?.includes(' ')
        ? String(value).split(' ')
        : [null, value]; // will be used only for tel
    const isCountryCodeRequired = preset === InputPresets.Tel;
    const commonInputProps = {
        fullWidth: isFullWidth,
        placeholder: placeholder,
        required: required,
        variant: variant,
    };
    const errorClass = isCountryCodeRequired
        ? ''
        : isErrorBorder
            ? 'adminInput__valueErrorWithBorder'
            : 'adminInput__valueError';
    const inputContainerClass = `adminInput__container ${containerClass} ${label === INPUT_LABELS.CONTACT_NUMBER && isCountryCodeRequired ? 'adminInput__wrapper' : ''}`;
    const textFieldClass = `adminInput__field ${inputFieldClass} ${hasBorder ? `adminInput__fieldWithBorder ${inputFieldWithBorderClass}` : ''} ${error ? errorClass : ''}`;
    const labelClass = `adminInput__label ${inputLabelClass} ${isLabelError ? 'adminInput__labelError' : ''} ${hasBorder ? `adminInput__labelWithBorder ${inputLabelWithBorderClass}` : 'adminInput__labelWithoutBorder'}`;
    const handleCountryCodeChange = (event) => {
        const newCountryCode = event.target.value;
        const res = `${newCountryCode} ${phoneNumber}`;
        onChange(res);
    };
    const handleShowPasswordClick = () => setShowPassword(show => !show);
    function parseValue(value) {
        return typeof value === 'string' && PARSE_DECIMAL_REGEX.test(value) ? value : Number(value);
    }
    const handleInputChange = (event) => {
        let finalValue = event.target.value;
        if (label === INPUT_LABELS.CONTACT_NUMBER && !finalValue.match(ONLY_NUMBER_REGEX)) {
            return;
        }
        if (countryCode && isCountryCodeRequired) {
            finalValue = `${countryCode} ${event.target.value}`;
        }
        if (preset === InputPresets.Number) {
            if (!finalValue.match(ONLY_NUMBER_REGEX)) {
                return;
            }
            const numericValue = numberType === NumberType.Float ? parseValue(finalValue) : Number(finalValue);
            onChange(numericValue);
        }
        else {
            !NO_SPACE_REGEX.test(finalValue) && onChange(finalValue);
        }
    };
    const getTextFieldComponent = (extraProps) => {
        const errorText = error && isErrorBorder && !isCountryCodeRequired ? helperText || 'Invalid Input' : '';
        const modifiedExtraProps = { ...extraProps };
        if (maxLength) {
            modifiedExtraProps.inputProps = {
                ...(extraProps.inputProps || {}),
                maxLength,
            };
        }
        return (_jsx(TextField, { name: name, ...modifiedExtraProps, ...commonInputProps, ...rest, disabled: disabled, autoComplete: autoComplete, className: textFieldClass, helperText: errorText, multiline: isMultiline, onChange: handleInputChange, value: isCountryCodeRequired ? phoneNumber : value, maxRows: maxLines }));
    };
    const renderComponent = () => {
        let component;
        const inputProps = variant === VariantTypes.Outlined ? {} : { disableUnderline: true };
        const iconPositionProp = iconPosition === InputIconPosition.Left
            ? {
                startAdornment: (_jsx(InputAdornment, { position: InputAdornmentPosition.Start, children: icon })),
            }
            : {
                endAdornment: (_jsx(InputAdornment, { position: InputAdornmentPosition.End, children: icon })),
            };
        switch (preset) {
            case InputPresets.Text:
                component = getTextFieldComponent({ InputProps: { ...inputProps, ...iconPositionProp } });
                break;
            case InputPresets.Number:
                component = getTextFieldComponent({
                    InputProps: {
                        ...inputProps,
                        ...iconPositionProp,
                    },
                    inputProps: { inputMode: 'numeric' },
                });
                break;
            case InputPresets.Email:
                component = getTextFieldComponent({
                    InputProps: { ...inputProps, ...iconPositionProp },
                    type: InputPresets.Email,
                });
                break;
            case InputPresets.Color:
                component = getTextFieldComponent({
                    InputProps: { ...inputProps, ...iconPositionProp },
                    type: InputPresets.Color,
                    value,
                });
                break;
            case InputPresets.Url:
                component = getTextFieldComponent({
                    InputProps: { ...inputProps, ...iconPositionProp },
                    type: InputPresets.Url,
                });
                break;
            case InputPresets.Tel:
                component = (_jsxs(_Fragment, { children: [renderNumberExtension(), getTextFieldComponent({
                            InputProps: { ...inputProps, ...iconPositionProp },
                            type: InputPresets.Tel,
                        })] }));
                break;
            case InputPresets.Password:
                inputProps.endAdornment = (_jsx(IconButton, { "aria-label": "Password Icon", className: "adminInput__passwordIcon", onClick: handleShowPasswordClick, children: showPassword ? _jsx(VisibilityOff, {}) : _jsx(Visibility, {}) }));
                component = getTextFieldComponent({
                    InputProps: inputProps,
                    type: showPassword ? InputType.Text : InputType.Password,
                });
                break;
            default:
                component = _jsx(_Fragment, {});
        }
        return component;
    };
    const renderItemValue = (selectedCountryCode) => {
        const selectedCountry = COUNTRIES.find(country => country.code === selectedCountryCode);
        return `${selectedCountry?.flag} ${selectedCountry?.code}`;
    };
    const renderNumberExtension = () => {
        const renderMenuItem = (country) => (_jsxs(MenuItem, { value: country.code, children: [country.flag, " ", country.name, " ", country.code] }, `${country.name} ${country.code}`));
        return (_jsxs(_Fragment, { children: [_jsx(Select, { value: countryCode ?? '', onChange: handleCountryCodeChange, className: "adminInput__dropdown", renderValue: renderItemValue, children: COUNTRIES.map(renderMenuItem) }), _jsx("div", { className: "adminInput__verticalLine" })] }));
    };
    return (_jsxs(Container, { className: inputContainerClass, children: [!!label && (_jsx("div", { className: `adminInput__labelContainer ${containerClass}`, children: _jsx(InputLabel, { error: error, className: labelClass, children: label }) })), _jsx("div", { className: isCountryCodeRequired
                    ? error
                        ? 'adminInput__countryCodeErrorWithBorder'
                        : 'adminInput__countryCodeContainer'
                    : 'adminInput__row', children: renderComponent() }), isCountryCodeRequired && error && (_jsx(Typography, { className: "adminInput__countryCodeErrorMessage", children: helperText }))] }));
};
export default React.memo(AdminInput);
