import { jsx as _jsx, Fragment as _Fragment, jsxs as _jsxs } from "react/jsx-runtime";
import React, { useMemo } from 'react';
import { Box, useMediaQuery, useTheme } from '@mui/material';
import { DataGrid } from '@mui/x-data-grid';
import { SearchIcon } from '../../assets';
import AdminInput from '../../components/input/AdminInput';
import { DEFAULT_COLUMN, InputPresets, InputSizes, TABLE_ACTIONS, TABLE_ACTIONS_KEYS, TableSortingOrder, } from '../../constants';
import { filterRowIfEmpty, renderCell } from '../../utils';
import AdminSpinner from '../spinner/AdminSpinner';
import getActions from './AdminTableActions';
import AdminTableFooter from './AdminTableFooter';
import NoResultFound from './NoResultFound';
import './adminTable-styles.scss';
import MobileTable from './mobile-table/MobileTable';
function AdminTable(props) {
    const { actions, columns, currentPage = 1, handleActionClick = () => { }, handlePageChange = () => { }, handleRowClick = () => { }, handleRowsPerPageChange = () => { }, handleSort = () => { }, handleToggle = () => { }, isExport, isLoading = false, toggleKey, pageSize = 10, pagination = false, rows, title = 'data', totalRecords = 0, toolbarHeader = null, handleSearch = () => { }, searchValue = '', disableActionsForRow = () => false, customActions = [], ...rest } = props;
    const theme = useTheme();
    const isMobile = useMediaQuery(theme.breakpoints.down('sm'));
    //  The filteredActions variable is created by filtering the TABLE_ACTIONS array and returning only the actions that are active.
    const filteredActions = actions && TABLE_ACTIONS.filter(action => actions[action.key]);
    const updatedActions = filteredActions?.filter(action => action.key !== TABLE_ACTIONS_KEYS.ADD);
    const isActionActive = updatedActions && updatedActions.length > 0;
    //  The updatedHeaders variable is created by mapping over the columns prop and adding the DEFAULT_COLUMN object to each column object.
    let updatedHeaders = columns?.map((column) => {
        const isWidthGiven = column.width && column.width > 0;
        return {
            ...(isWidthGiven
                ? {
                    ...DEFAULT_COLUMN,
                }
                : {
                    ...DEFAULT_COLUMN,
                    flex: 1,
                }),
            ...column,
            renderCell: column.renderCell
                ? column.renderCell
                : params => renderCell(params, column),
            valueGetter: (params) => {
                const updatedValue = !params ? '-' : params;
                return updatedValue;
            },
        };
    });
    //  If the isActionActive variable is true, an additional column object is added to the updatedHeaders array.
    if (isActionActive) {
        updatedHeaders = [
            ...updatedHeaders,
            {
                ...DEFAULT_COLUMN,
                width: 150,
                field: 'actions',
                type: 'actions',
                align: 'right',
                getActions: getActions([...updatedActions, ...(customActions ?? [])], handleActionClick, handleToggle, toggleKey, disableActionsForRow),
            },
        ];
    }
    const columnVisibility = updatedHeaders.reduce((acc, column) => {
        acc[column.field] = column.isVisible ?? true;
        return acc;
    }, {});
    const memoizedRows = useMemo(() => filterRowIfEmpty(rows), [rows]);
    return (_jsxs(Box, { className: "adminTable__tableContainer", children: [actions && actions[TABLE_ACTIONS_KEYS.SEARCH] && (_jsx(AdminInput, { placeholder: "Search...", inputFieldClass: "adminTable__searchField", size: InputSizes.Medium, onChange: handleSearch, icon: _jsx(SearchIcon, {}), value: searchValue, preset: InputPresets.Text })), isMobile ? (_jsx(MobileTable, { ...props, rows: memoizedRows })) : (_jsx(DataGrid, { classes: {
                    cell: 'adminTable__rowCell',
                    columnHeader: 'adminTable__headers',
                    root: 'adminTable__container',
                    row: 'adminTable__row',
                }, columns: updatedHeaders, disableColumnFilter: true, disableColumnMenu: true, disableColumnSelector: true, disableRowSelectionOnClick: true, sortingOrder: [TableSortingOrder.ASC, TableSortingOrder.DESC], onSortModelChange: handleSort, rows: rows, initialState: {
                    columns: {
                        columnVisibilityModel: columnVisibility,
                    },
                }, loading: isLoading, onRowClick: handleRowClick, sortingMode: "server", slots: {
                    footer: () => (_jsx(AdminTableFooter, { currentPage: currentPage, onPageChange: handlePageChange, onRowsPerPageChange: handleRowsPerPageChange, pageSize: pageSize, title: title, totalRecords: totalRecords, isExport: isExport, isPagination: pagination })),
                    noRowsOverlay: () => _jsx(NoResultFound, {}),
                    loadingOverlay: () => _jsx(AdminSpinner, {}),
                    toolbar: () => _jsx(_Fragment, { children: toolbarHeader }),
                }, ...rest }))] }));
}
export default React.memo(AdminTable);
